@@ -18,10 +18,7 @@
 # Accounts interface for Gwibber
 #
 
-try:
-  import gnomekeyring
-except:
-  gnomekeyring = None
+from gi.repository import Gdk, GdkPixbuf, Gtk, Pango
 
 import gettext
 from gettext import lgettext as _
@@ -29,15 +26,16 @@ if hasattr(gettext, 'bind_textdomain_codeset'):
   gettext.bind_textdomain_codeset('gwibber','UTF-8')
 gettext.textdomain('gwibber')
 
-import gtk, pango, os, json, uuid
+import os, json, uuid
 from gwibber.lib import GwibberPublic
-import util
-from microblog.util import resources
+from gwibber.util import get_service_icon
+from gwibber.microblog.util import keyring, resources
 
 for p in resources.get_plugin_dirs()[0]:
   print "Loading plugin for %s" % p
   globals()[p] = __import__("%s.gtk.%s" % (p,p), fromlist='*')
 
+
 from dbus.mainloop.glib import DBusGMainLoop
 DBusGMainLoop(set_as_default=True)
 
@@ -48,24 +46,31 @@ settings_interface = "org.gwibber.preferences"
 
 class GwibberAccountManager(object):
   def __init__(self, selected_account=None, condition=None, message=None):
-    self.selected_account = selected_account
+    self.icon_theme = Gtk.IconTheme.get_default ()
+    self.icon_theme.prepend_search_path ("/usr/share/gwibber/ui/icons/")
+    if os.path.exists(os.path.join("data", "icons")):
+      self.icon_theme.prepend_search_path (os.path.realpath(os.path.join("data", "icons")))
+
+    self.selected_account = []
+    if selected_account:
+      self.selected_account = selected_account.split(",")
     self.condition = condition
     self.message = message
     self.gwibber = GwibberPublic()
     self.services = json.loads(self.gwibber.GetServices())
-    self.ui = gtk.Builder()
+    self.ui = Gtk.Builder()
     self.ui.set_translation_domain("gwibber")
     self.ui.add_from_file(resources.get_ui_asset("gwibber-accounts-dialog.ui"))
     self.ui.connect_signals(self)
 
-    dialog = self.ui.get_object("accounts_dialog")
-    dialog.resize(640,400)
-    dialog.set_icon_name("gwibber")
-    dialog.set_wmclass("gwibber", "Gwibber")
-    dialog.show_all()
-    if hasattr(gtk, "InfoBar"):
+    self.dialog = self.ui.get_object("accounts_dialog")
+    self.dialog.resize(640,400)
+    self.dialog.set_icon_name("gwibber")
+    self.dialog.set_wmclass("gwibber", "Gwibber")
+    self.dialog.show_all()
+    if hasattr(Gtk, "InfoBar"):
       self.alignment_infobar = self.ui.get_object("alignment_infobar")
-      self.infobar = gtk.InfoBar()
+      self.infobar = Gtk.InfoBar()
       self.alignment_infobar.add(self.infobar)
       self.infobar.hide()
 
@@ -80,11 +85,10 @@ class GwibberAccountManager(object):
     
     self.setup_account_tree()
     self.populate_account_tree()
-    self.select_account()
     
   def setup_account_tree(self):
     # Protocol title, Icon, Protocol Properties
-    self.service_store = gtk.ListStore(str, gtk.gdk.Pixbuf, str)
+    self.service_store = Gtk.ListStore(str, GdkPixbuf.Pixbuf, str)
     for name, props in self.services.items():
       icon = self.get_icon(name)
       if icon:
@@ -93,8 +97,8 @@ class GwibberAccountManager(object):
     service_combo = self.ui.get_object("protocol_combobox")
     service_combo.set_model(self.service_store)
 
-    celltxt = gtk.CellRendererText()
-    cellimg = gtk.CellRendererPixbuf()
+    celltxt = Gtk.CellRendererText()
+    cellimg = Gtk.CellRendererPixbuf()
     
     service_combo.pack_start(cellimg, False)
     service_combo.pack_start(celltxt, False)
@@ -103,13 +107,13 @@ class GwibberAccountManager(object):
     service_combo.set_active(0)
 
     # Store: Account name, Icon, Protocol properties, Color
-    self.account_store = gtk.TreeStore(str, gtk.gdk.Pixbuf, object, str)
+    self.account_store = Gtk.TreeStore(str, GdkPixbuf.Pixbuf, object, str)
 
-    icon_column = gtk.TreeViewColumn('Icon')
-    name_column = gtk.TreeViewColumn('Account')
+    icon_column = Gtk.TreeViewColumn('Icon')
+    name_column = Gtk.TreeViewColumn('Account')
 
-    celltxt = gtk.CellRendererText()
-    cellimg = gtk.CellRendererPixbuf()
+    celltxt = Gtk.CellRendererText()
+    cellimg = Gtk.CellRendererPixbuf()
 
     icon_column.pack_start(cellimg, False)
     icon_column.add_attribute(cellimg, 'pixbuf', 1)
@@ -125,39 +129,29 @@ class GwibberAccountManager(object):
     self.account_tree.set_model(self.account_store)
 
   def select_account(self):
-    if not self.selected_account:
+    column = self.account_tree.get_column(0)
+    if len(self.selected_account) < 1:
       if len(self.account_store):
-        self.account_tree.set_cursor(0)
+        self.account_tree.set_cursor(Gtk.TreePath.new_from_string('0'),
+                                     column, False)
       else: self.add_account()
     else:
       for i in self.account_store:
-        if i[2]["id"] == self.selected_account:
-          self.account_tree.set_cursor(i.path)
-      #self.selected_account = None
-
+        if len(self.selected_account) > 0:
+          if i[2]["id"] == self.selected_account[0]:
+            self.account_tree.set_cursor(i.path, column, False)
 
   def get_icon(self, name):
-    icf = util.get_service_icon(name)
-    if icf:
-      if os.path.exists(icf):
-        return gtk.gdk.pixbuf_new_from_file(icf)
-    #return gtk.gdk.pixbuf_new_from_file(resources.get_ui_asset("gwibber.png"))
-    return None
-
-  def get_from_keyring(self, acctid, name):
-    item = {"id": str("%s/%s" % (acctid, name))}
     try:
-      return gnomekeyring.find_items_sync(gnomekeyring.ITEM_GENERIC_SECRET, item)[0].secret
-    except gnomekeyring.NoMatchError:
-      return None
-
-  def put_in_keyring(self, acctid, name, value):
-    id = "%s/%s" % (acctid, name)
-    return gnomekeyring.item_create_sync(
-        gnomekeyring.get_default_keyring_sync(),
-        gnomekeyring.ITEM_GENERIC_SECRET,
-        "Gwibber preference: %s" % id,
-        {"id": str(id)}, str(value), True)
+      icon = self.icon_theme.load_icon (name, 16, 0)
+    except:
+      icon_path = os.path.join("gwibber", "microblog", "plugins", name, "ui", "icons")
+      if os.path.exists(icon_path):
+        self.icon_theme.prepend_search_path (os.path.realpath(icon_path))
+        icon = self.icon_theme.load_icon (name, 16, 0)
+      else:
+        icon = self.icon_theme.load_icon ("gwibber", 16, 0)
+    return icon
 
   def populate_account_tree(self):
     self.account_store.clear()
@@ -172,14 +166,18 @@ class GwibberAccountManager(object):
           private_fields = [f.split(":")[-1] for f in self.services[account["service"]]["config"] if ":" in f]
           color = None
           for f in private_fields:
-            if not account.has_key(f): account[f] = ":KEYRING:MISSING"
+            if not account.has_key(f) or account[f] is None: account[f] = ":KEYRING:MISSING"
             if account[f].startswith(":KEYRING:"):
-              value = self.get_from_keyring(account["id"], f)
-              if value is None:
+              value = keyring.get_from_keyring(account["id"], f)
+              if value is None or account[f] == ":KEYRING:MISSING":
                 color = "pink"
+                if account["id"] not in self.selected_account:
+                  self.selected_account.append (account["id"])
+                  self.condition = "error"
           self.account_store.append(None, [name, icon, account, color])
         except:
           pass
+    self.select_account()
 
   def verify_account(self, account):
     for field in self.services[account["service"]]["config"]:
@@ -205,24 +203,22 @@ class GwibberAccountManager(object):
       pass
     else:  
       if "id" in self.account:
+        print self.account['id']
         try:
           private_fields = [f.split(":")[-1] for f in self.services[self.account["service"]]["config"] if ":" in f]
           for f in private_fields:
             if self.account[f].startswith(":KEYRING:"):
               keyring_id = int(self.account[f].split(":")[-1])
-              gnomekeyring.item_delete_sync(gnomekeyring.get_default_keyring_sync(), keyring_id)
+              keyring.del_from_keyring(keyring_id)
         except:
           pass  
         self.gwibber.accounts.Delete(self.account["id"])
       self.populate_account_tree()
-      self.select_account()
 
   def on_edit_account_save(self, widget=None):
-    print "Saving..."
-    
     service = self.services[self.account["service"]]
     is_new = "id" not in self.account
-      
+
     if is_new: self.account["id"] = uuid.uuid1().hex
     
     for config in service["config"]:
@@ -230,7 +226,6 @@ class GwibberAccountManager(object):
       config = config.replace("private:", "")
       widget = self.account_widget.ui.get_object(config)
 
-
       value = None
 
       for p in ["text", "active"]:
@@ -238,20 +233,18 @@ class GwibberAccountManager(object):
           value = getattr(widget.props, p)
           self.account[config] = value
 
-      if widget and isinstance(widget, gtk.ColorButton):
-        value = widget.get_color ()
-        self.account[config] = value
+      if widget and isinstance(widget, Gtk.ColorButton):
+        value = widget.get_color()
+        self.account[config] = value.to_string()
 
       if is_private and self.account.has_key(config):
         if not self.account[config].startswith(":KEYRING:"):
-	  self.account[config] = ":KEYRING:%s" % self.put_in_keyring(
+	  self.account[config] = ":KEYRING:%s" % keyring.put_in_keyring(
 	      self.account["id"], config, self.account[config])
 
       if value:
-        if isinstance(value, gtk.gdk.Color):
-          value = gtk.color_selection_palette_to_string(
-            gtk.color_selection_palette_from_string(value.to_string()))
-          self.account[config] = value
+        if isinstance(value, Gdk.Color):
+          self.account[config] = value.to_string()
 
     if not self.verify_account(self.account): return False
 
@@ -262,10 +255,10 @@ class GwibberAccountManager(object):
     self.ui.get_object('vbox_create').hide()
 
     # if we just fixed a failure case, hide the infobar
-    if self.account["id"] == self.selected_account:
-      if hasattr(gtk, "InfoBar"):
+    if self.account["id"] in self.selected_account:
+      if hasattr(Gtk, "InfoBar") and len(self.selected_account) <= 1:
         self.infobar.hide()
-      self.selected_account = None
+      self.selected_account.remove (self.account["id"])
 
     self.populate_account_tree()
 
@@ -299,18 +292,18 @@ class GwibberAccountManager(object):
     self.account_show(service, icon)
 
   def account_show(self, service_id, icon=None, account=None, condition=None, message=None):
-    if hasattr(gtk, "InfoBar"):
+    if hasattr(Gtk, "InfoBar"):
       if condition == "error":
-        self.infobar.set_message_type(gtk.MESSAGE_ERROR)
+        self.infobar.set_message_type(Gtk.MessageType.ERROR)
       if condition == "info":
-        self.infobar.set_message_type(gtk.MESSAGE_INFO)
+        self.infobar.set_message_type(Gtk.MessageType.INFO)
       if message and condition:
         if hasattr(self, "infobar_content_area"):
           for child in self.infobar_content_area.get_children(): child.destroy()
         self.infobar_content_area = self.infobar.get_content_area()
-        message_label = gtk.Label(message)
+        message_label = Gtk.Label(message)
         message_label.set_use_markup(True)
-        message_label.set_ellipsize(pango.ELLIPSIZE_END)
+        message_label.set_ellipsize(Pango.EllipsizeMode.END)
         self.infobar_content_area.add(message_label)
         self.infobar.show_all()
       else:
@@ -327,7 +320,7 @@ class GwibberAccountManager(object):
     self.ui.get_object('label_name').set_label(service["name"])
 
     if icon:
-      self.ui.get_object('image_type').set_from_pixbuf(icon)
+      self.ui.get_object('image_type').set_from_pixbuf(self.get_icon(service_id))
 
     if service_id in globals():
       if not account:
@@ -338,7 +331,7 @@ class GwibberAccountManager(object):
 
       aw = globals()[service_id].AccountWidget(account, self)
 
-      vbox_account.pack_start(aw, False, False)
+      vbox_account.pack_start(aw, False, False, 0)
       vbox_account.show()
 
       if "color" not in account:
@@ -352,21 +345,22 @@ class GwibberAccountManager(object):
           value = account[config]
           widget = aw.ui.get_object(config)
           
-          if isinstance(widget, gtk.Entry):
+          if isinstance(widget, Gtk.Entry):
             if is_private and account[config].startswith(":KEYRING:"):
-              value = self.get_from_keyring(account["id"], config)
+              value = keyring.get_from_keyring(account["id"], config)
               if not value:
-                widget.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("pink"))
+                widget.modify_base(Gtk.StateType.NORMAL, Gdk.Color.parse("pink")[1])
             if value:
               widget.set_text(value)
             widget.connect("changed", self.on_edit_account)
 
-          elif isinstance(widget, gtk.CheckButton):
+          elif isinstance(widget, Gtk.CheckButton):
             widget.set_active(value)
             widget.connect("toggled", self.on_edit_account)
 
-          elif isinstance(widget, gtk.ColorButton):
-            widget.set_color(gtk.color_selection_palette_from_string(value)[0])
+          elif isinstance(widget, Gtk.ColorButton):
+            parsed, color = Gdk.Color.parse(value)
+            widget.set_color(color)
             widget.connect("color-set", self.on_edit_account)
 
       self.account = account
@@ -375,22 +369,25 @@ class GwibberAccountManager(object):
       self.ui.get_object('vbox_create').hide()
 
   def on_accounts_dialog_destroy(self, widget, data=None):
-    gtk.main_quit()
+    Gtk.main_quit()
 
   def on_accounts_tree_row_activated(self, widget, data=None):
     self.ui.get_object('vbox_save').hide()
     self.ui.get_object('vbox_create').hide()
-    
-    model, rows = widget.get_selection().get_selected_rows()
 
-    if rows:
-      iter = model.get_iter(rows[0][0])
+    selection = widget.get_selection()
+    rows, model = (None, None)
+    if selection:
+      model, rows = selection.get_selected_rows()
+
+    if rows and model:
+      iter = model.get_iter(rows[0])
       icon = model.get_value(iter, 1)
       acct = model.get_value(iter, 2)
-      if self.selected_account == acct["id"] and self.condition:
+      if acct["id"] in self.selected_account and self.condition:
         self.account_show(acct["service"], icon, acct, self.condition, self.message)
       else:
         self.account_show(acct["service"], icon, acct)
 
   def on_button_close_clicked(self, widget, data=None):
-    gtk.main_quit()
+    Gtk.main_quit()
