@@ -1,20 +1,22 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-import multiprocessing, threading, traceback, json
-import gobject, dbus, dbus.service
+import traceback, json, random, string
+import dbus, dbus.service
 import sqlite3, mx.DateTime, re, uuid
 import urlshorter, storage, network, util, uploader
 from gettext import lgettext as _
-import signal
 
-from util import log
-from util import resources
-from util import exceptions
+from gi.repository import GLib
+from threading import Thread
+from datetime import datetime, timedelta
+
+import logging
+logger = logging.getLogger("Dispatcher")
+
 from util.const import *
 import subprocess
 
-
 try:
   from gi.repository import Unity, Dbusmenu
 except:
@@ -35,16 +37,16 @@ try:
 except:
   indicate = None
 
-gobject.threads_init()
+GLib.threads_init()
 
 # Dynamically build a list of available service plugins
 PROTOCOLS = {}
 for p in util.resources.get_plugin_dirs()[0]:
     PROTOCOLS[str(p)] = __import__("%s" % p, fromlist='*')
-    print "Loading plugin %s version %s" % (PROTOCOLS[str(p)].PROTOCOL_INFO["name"], PROTOCOLS[str(p)].PROTOCOL_INFO["version"])
+    # print "Loading plugin %s version %s" % (PROTOCOLS[str(p)].PROTOCOL_INFO["name"], PROTOCOLS[str(p)].PROTOCOL_INFO["version"])
     #print "Path %s" % str(PROTOCOLS[str(p)].__file__)
     # FIXME: Figure out why the logger doesn't log here
-    #log.logger.info("Loading plugin for %s", p)
+    #logger.info("Loading plugin for %s", p)
 
 FEATURES = json.loads(GWIBBER_OPERATIONS)
 SERVICES = dict([(k, v.PROTOCOL_INFO) for k, v in PROTOCOLS.items()])
@@ -54,71 +56,85 @@ gsettings = Gio.Settings.new("org.gwibber.preferences")
 if gsettings.get_int("interval") < 5:
   gsettings.set_int("interval", 5)
 
-log.logger.name = "Gwibber Dispatcher"
+class perform_operation(Thread):
+  def __init__(self, job, callback_success, callback_failure):
+    Thread.__init__(self)
+    self.job = job
+    self.t_start = None
+    self.t_end = None
+    self.t_runtime = timedelta(0)
+    self.callback_success = callback_success
+    self.callback_failure = callback_failure
 
-def perform_operation((account, opname, args, transient)):
-  try:
-    stream = FEATURES[opname]["stream"] or opname
-    logtext = "<%s:%s>" % (account["service"], opname)
+  def run(self):
+    self.id = ''.join(random.choice(string.letters) for i in xrange(5))
+    self.t_start = datetime.now()
+    (account, opname, args, transient) = self.job
+    try:
+      stream = FEATURES[opname]["stream"] or opname
+      logtext = "<%s:%s>" % (account["service"], opname)
 
-    logtext = "<%s:%s>" % (account["service"], opname)
-    log.logger.debug("%s Performing operation", logtext)
+      logtext = "<%s:%s>" % (account["service"], opname)
+      logger.debug("%s Performing operation", logtext)
 
-    args = dict((str(k), v) for k, v in args.items())
-    message_data = PROTOCOLS[account["service"]].Client(account)(opname, **args)
-    text_cleaner = re.compile(u"[: \n\t\r♻♺]+|@[^ ]+|![^ ]+|#[^ ]+") # signs, @nickname, !group, #tag
-    new_messages = []
+      args = dict((str(k), v) for k, v in args.items())
+      message_data = PROTOCOLS[account["service"]].Client(account)(opname, **args)
+      text_cleaner = re.compile(u"[: \n\t\r♻♺]+|@[^ ]+|![^ ]+|#[^ ]+") # signs, @nickname, !group, #tag
+      new_messages = []
 
-    if message_data is not None:
-      for m in message_data:
-        try: 
-          if isinstance(m, dict) and m.has_key("mid"):
-            m["id"] = uuid.uuid1().hex
-            m["operation"] = opname
-            m["stream"] = m.get("stream", stream)
-            m["transient"] = transient
-            m["time"] = m.get("time", 0)
-            if not m["text"]: m["text"] = ""
-            m["rtl"] = util.isRTL(re.sub(text_cleaner, "", m["text"].decode('utf-8')))
-            if m.has_key("type"):
-              if m["type"] == "link": m["stream"] = "links"
-              if m["type"] == "video": m["stream"] = "videos"
-              if m["type"] == "photo": m["stream"] = "images"
+      if message_data is not None:
+        for m in message_data:
+          try:
+            if isinstance(m, dict) and m.has_key("mid"):
+              m["id"] = uuid.uuid1().hex
+              m["operation"] = opname
+              m["stream"] = m.get("stream", stream)
+              m["transient"] = transient
+              m["time"] = m.get("time", 0)
+              if not m["text"]: m["text"] = ""
+              m["rtl"] = util.isRTL(re.sub(text_cleaner, "", m["text"]))
+              if m.has_key("type"):
+                if m["type"] == "link": m["stream"] = "links"
+                if m["type"] == "video": m["stream"] = "videos"
+                if m["type"] == "photo": m["stream"] = "images"
 
-            log.logger.debug("%s Adding record", logtext)
+              logger.debug("%s Adding record", logtext)
+              new_messages.insert(0, (
+                                  m["id"],
+                                  m["mid"],
+                                  m["account"],
+                                  account["service"],
+                                  opname,
+                                  transient,
+                                  m["stream"] or stream,
+                                  m["time"],
+                                  m["text"],
+                                  m.get("sender", {}).get("is_me", None),
+                                  m.get("to_me", None),
+                                  m.get("sender", {}).get("nick", None),
+                                  m.get("reply", {}).get("nick", None),
+                                  json.dumps(m)
+              ))
+            elif isinstance(m, dict) and m.has_key("error"):
+              new_messages.insert(0, (
+                                  "error",
+                                  json.dumps(m)
+              ))
+          except Exception as e:
+            if not "logtext" in locals(): logtext = "<UNKNOWN>"
+            logger.error("%s Operation failed: %s", logtext, e)
+      self.callback_success(new_messages)
 
-            new_messages.insert(0, (
-              m["id"],
-              m["mid"],
-              m["account"],
-              account["service"],
-              opname,
-              transient,
-              m["stream"] or stream,
-              m["time"],
-              m["text"],
-              m.get("sender", {}).get("is_me", None), 
-              m.get("to_me", None),
-              m.get("sender", {}).get("nick", None),
-              m.get("reply", {}).get("nick", None),
-              json.dumps(m)
-            ))
-          elif isinstance(m, dict) and m.has_key("error"):
-            new_messages.insert(0, (
-              "error",
-              json.dumps(m)
-            ))
-        except Exception as e:
-          if not "logtext" in locals(): logtext = "<UNKNOWN>"
-          log.logger.error("%s Operation failed: %s", logtext, e)
+    except Exception as e:
+      if not "logtext" in locals(): logtext = "<UNKNOWN>"
+      logger.error("%s Operation failed", logtext)
+      logger.debug("Traceback:\n%s", traceback.format_exc())
+      self.callback_failure("Error")
+
+    self.t_end = datetime.now()
+    self.t_runtime = self.t_end - self.t_start
+    logger.debug("%s Finished operation (%s)" % (logtext, str(self.t_runtime)))
 
-    log.logger.debug("%s Finished operation", logtext)
-    return ("Success", new_messages)
-  except Exception as e:
-    if not "logtext" in locals(): logtext = "<UNKNOWN>"
-    log.logger.error("%s Operation failed", logtext)
-    log.logger.debug("Traceback:\n%s", traceback.format_exc())
-    return ("Failure", traceback.format_exc())
 
 class OperationCollector:
   def __init__(self, dispatcher):
@@ -165,11 +181,11 @@ class OperationCollector:
   def validate_operation(self, acct, opname, enabled="receive_enabled"):
     # if account doesn't have the required feature or is disabled, return
     if enabled in acct:
-      if not acct[enabled]: return
+      if not acct[enabled]: return False
     else: 
-      return
+      return False
     # if there is an account for a service that gwibber doesn't no about, return
-    if not acct["service"] in SERVICES: return
+    if not acct["service"] in SERVICES: return False
     service = SERVICES[acct["service"]]
     return acct["service"] in PROTOCOLS and \
            opname in service["features"] and \
@@ -189,7 +205,7 @@ class OperationCollector:
 
   def search_to_operations(self, search):
     for account in self.get_accounts():
-      args = {"query": search["query"]}
+      args = {"query": search["query"].encode("utf-8")}
       if self.validate_operation(account, "search"):
         args.update(self.handle_max_id(account, "search", search["id"]))
         yield (account, "search", args, search["id"])
@@ -230,23 +246,6 @@ class OperationCollector:
       for o in self.search_to_operations(search):
         yield o
 
-class MapAsync(threading.Thread):
-  def __init__(self, func, iterable, cbsuccess, cbfailure, pool):
-    threading.Thread.__init__(self)
-    self.iterable = iterable
-    self.callback = cbsuccess
-    self.failure = cbfailure
-    self.daemon = True
-    self.func = func
-    self.pool = pool
-    self.start()
-
-  def run(self):
-    try:
-      self.pool.map_async(self.func, self.iterable, callback = self.callback)
-    except Exception as e:
-      self.failure(e, traceback.format_exc())
-
 class Dispatcher(dbus.service.Object):
   """
   The Gwibber Dispatcher handles all the backend operations.
@@ -272,29 +271,6 @@ class Dispatcher(dbus.service.Object):
     self.connection_monitor.connect_to_signal("ConnectionOnline", self.on_connection_online)
     self.connection_monitor.connect_to_signal("ConnectionOffline", self.on_connection_offline)
 
-    self.indicate = None 
-
-    if indicate and util.resources.get_desktop_file():
-      self.indicate = indicate.Server.ref_default ()
-      self.indicate.set_type("message.gwibber")
-      self.indicate.set_desktop_file(util.resources.get_desktop_file())
-      self.indicate.connect("server-display", self.on_indicator_server_activate)
-      self.indicate.connect("interest-added", self.on_indicator_interest_added)
-      self.indicate.connect("interest-removed", self.on_indicator_interest_removed)
-      self.indicate.show()
-
-    if Dbusmenu:
-      post_menu = Dbusmenu.Menuitem.new ()
-      post_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Update Status"))
-      post_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
-      post_menu.connect("item-activated", self.show_poster)
-
-      menu_server = Dbusmenu.Server.new("/messaging/commands")
-      root = Dbusmenu.Menuitem.new ()
-      root.child_append (post_menu)
-      menu_server.set_root(root);
-      self.indicate.set_menu (menu_server)
-
     self.indicator_items = {}
     self.notified_items = []
     self.notified_errors = {}
@@ -305,6 +281,32 @@ class Dispatcher(dbus.service.Object):
     for s in "messages", "replies", "private":
       self.unseen_counts[s] = 0
 
+    self.indicate = None 
+    self.launcher = None
+
+    self.job_list = []
+
+    if indicate and util.resources.get_desktop_file():
+      self.indicate = indicate.Server.ref_default ()
+      self.indicate.set_type("message.gwibber")
+      self.indicate.set_desktop_file(util.resources.get_desktop_file())
+      if Dbusmenu and self.indicate:
+        post_menu = Dbusmenu.Menuitem.new ()
+        post_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Update Status"))
+        post_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
+        post_menu.connect("item-activated", self.show_poster)
+
+        menu_server = Dbusmenu.Server.new("/messaging/commands")
+        root = Dbusmenu.Menuitem.new ()
+        root.child_append (post_menu)
+        menu_server.set_root(root)
+        self.indicate.set_menu (menu_server)
+      self.indicate.connect("server-display", self.on_indicator_server_activate)
+      self.indicate.connect("interest-added", self.on_indicator_interest_added)
+      self.indicate.connect("interest-removed", self.on_indicator_interest_removed)
+      self.update_indicators(self.unseen_counts)
+      self.indicate.show()
+
     if Unity and Dbusmenu:
       self.launcher = Unity.LauncherEntry.get_for_desktop_id ("gwibber.desktop")
       ql = Dbusmenu.Menuitem.new ()
@@ -339,7 +341,6 @@ class Dispatcher(dbus.service.Object):
       self.launcher.set_property("count_visible", False)
 
     self.refresh_count = 0
-    self.workerpool = multiprocessing.Pool()
 
     self.refresh_timer_id = None
 
@@ -349,9 +350,9 @@ class Dispatcher(dbus.service.Object):
 
     if autorefresh:
       if self.refresh_timer_id:
-        gobject.source_remove(self.refresh_timer_id)
+        GLib.source_remove(self.refresh_timer_id)
       # wait a few seconds before alerting the world we are online
-      self.refresh_timer_id = gobject.timeout_add_seconds(int(10), self.refresh)
+      self.refresh_timer_id = GLib.timeout_add_seconds(int(10), self.refresh)
 
     self.accounts_service = util.getbus("Accounts")
     self.accounts_service.connect_to_signal("Updated", self.on_account_updated)
@@ -388,9 +389,9 @@ class Dispatcher(dbus.service.Object):
   def do_maintenance(self, *args):
     # perform some needed MessageManager maintenance
     if self.maint_timer_id:
-      gobject.source_remove(self.maint_timer_id)
+      GLib.source_remove(self.maint_timer_id)
     if self.refreshRunning:
-      self.maint_timer_id = gobject.timeout_add_seconds(60, self.do_maintenance)
+      self.maint_timer_id = GLib.timeout_add_seconds(60, self.do_maintenance)
       return False
 
     self.maintRunning = True
@@ -400,20 +401,20 @@ class Dispatcher(dbus.service.Object):
     return False
     
   def on_connection_online(self, *args):
-    log.logger.info("Dispatcher Online, initiating a refresh")
+    logger.info("Dispatcher Online, initiating a refresh")
     if self.refresh_timer_id:
-      gobject.source_remove(self.refresh_timer_id)
+      GLib.source_remove(self.refresh_timer_id)
     # wait a few seconds before alerting the world we are online
-    self.refresh_timer_id = gobject.timeout_add_seconds(int(10), self.refresh)
+    self.refresh_timer_id = GLib.timeout_add_seconds(int(10), self.refresh)
 
   def on_connection_offline(self, *args):
     self.refreshRunning = False
-    log.logger.info("Dispatcher Offline, suspending operations")
+    logger.info("Dispatcher Offline, suspending operations")
     if self.refresh_timer_id:
-      gobject.source_remove(self.refresh_timer_id)
+      GLib.source_remove(self.refresh_timer_id)
 
   def on_account_updated(self, account):
-    pass
+    self.refresh()
 
   def on_account_created(self, account):
     self.refresh()
@@ -462,7 +463,7 @@ class Dispatcher(dbus.service.Object):
     try: o = json.loads(opdata)
     except: return
     
-    log.logger.debug("** Starting Single Operation **")
+    logger.debug("** Starting Single Operation **")
     self.LoadingStarted()
     
     params = ["account", "operation", "args", "transient"]
@@ -502,12 +503,14 @@ class Dispatcher(dbus.service.Object):
             service = dbus.Interface(obj, "com.Gwibber.Service")
             service.SendMessage("Your message")
     """
-    self.send(list(self.collector.get_send_operations(message)))
+    self.send(list(self.collector.get_send_operations(message.encode('utf-8'))))
 
   @dbus.service.method("com.Gwibber.Service", in_signature="s")
   def Send(self, opdata):
     try:
+      operations = []
       o = json.loads(opdata)
+      o["message"] = o["message"].encode("utf-8")
       if "target" in o:
         args = {"message": o["message"], "target": o["target"]}
         operations = [(self.collector.get_account(o["target"]["account"]), "send_thread", args, None)]
@@ -518,11 +521,11 @@ class Dispatcher(dbus.service.Object):
         operations = [(self.collector.get_account(a), "send", {"message": o["message"]}, None) for a in o["accounts"]]
       self.send(operations)
     except:
-      log.logger.error("Sending failed:\n%s", traceback.format_exc())
+      logger.error("Sending failed:\n%s", traceback.format_exc())
 
   @dbus.service.method("com.Gwibber.Service", in_signature="ss")
   def Retweet(self, mid, account):
-    log.logger.debug("Retweeting %s", mid)
+    logger.debug("Retweeting %s", mid)
     self.PerformOp(json.dumps({
       "account": account,
       "operation": "retweet",
@@ -532,7 +535,7 @@ class Dispatcher(dbus.service.Object):
 
   @dbus.service.method("com.Gwibber.Service", in_signature="ss")
   def Like(self, mid, account):
-    log.logger.debug("Liking %s", mid)
+    logger.debug("Liking %s", mid)
     self.PerformOp(json.dumps({
       "account": account,
       "operation": "like",
@@ -540,6 +543,16 @@ class Dispatcher(dbus.service.Object):
       "transient": False,
     }))
 
+  @dbus.service.method("com.Gwibber.Service", in_signature="ss")
+  def UnLike(self, mid, account):
+    logger.debug("Unliking %s", mid)
+    self.PerformOp(json.dumps({
+      "account": account,
+      "operation": "unlike",
+      "args": {"message": {"mid": mid}},
+      "transient": False,
+    }))
+
   @dbus.service.method("com.Gwibber.Service", out_signature="s")
   def GetServices(self):
     """
@@ -599,7 +612,7 @@ class Dispatcher(dbus.service.Object):
             service = dbus.Interface(obj, "com.Gwibber.Service")
             service.Start()
     """
-    log.logger.info("Gwibber Service is starting")
+    logger.info("Gwibber Service is starting")
 
   @dbus.service.method("com.Gwibber.Service")
   def Quit(self): 
@@ -611,7 +624,7 @@ class Dispatcher(dbus.service.Object):
             service = dbus.Interface(obj, "com.Gwibber.Service")
             service.Quit()
     """
-    log.logger.info("Gwibber Service is being shutdown")
+    logger.info("Gwibber Service is being shutdown")
     self.mainloop.quit()
 
   @dbus.service.method("com.Gwibber.Service", out_signature="b")
@@ -646,7 +659,7 @@ class Dispatcher(dbus.service.Object):
     
   def send_error_notify(self, error):
     if not isinstance(error, dict):
-      log.logger.error("Failed to parse error message: %s", error)
+      logger.error("Failed to parse error message: %s", error)
       return
     if error.has_key("error"):
       error = json.loads(error)["error"]
@@ -657,36 +670,57 @@ class Dispatcher(dbus.service.Object):
       if self.notified_errors[error["account"]["service"]] == error["message"]:
         return
     if util.can_notify:
-      icon = util.resources.get_ui_asset("icons/breakdance/scalable/%s.svg" % error["account"]["service"])
-      if not icon:
-        icon = util.resources.get_ui_asset("gwibber.svg")
       util.notify(error["account"]["service"], error["message"], icon, 2000)
     self.notified_errors[error["account"]["service"]] = error["message"]
 
-  def perform_async_operation(self, iterable):
-    t = MapAsync(perform_operation, iterable, self.loading_complete, self.loading_failed, self.workerpool)
-    t.join()
-  
-  def loading_complete(self, output):
+  def perform_async_operation (self, jobs):
+    #
+    # Clean old, not running jobs ...
+    #
+    logger.info("Running Jobs: %s", len(self.job_list))
+
+    for job in reversed(self.job_list):
+      if not job.isAlive():
+        job.join()
+        self.job_list.remove(job)
+
+    logger.info("Running Jobs: %s", len(self.job_list))
+
+    #
+    # Start all jobs
+    #
+    #
+    for job in jobs:
+      thread = perform_operation(job, self.cb_loading_complete, self.cb_loading_failed)
+      self.job_list.append(thread)
+      thread.start ()
+
+    logger.info("Running Jobs: %s", len(self.job_list))
+
+  #
+  # TODO: Make me prettier
+  #
+  def cb_loading_failed(self, error):
+      logger.info("Loading Error: %s - %s", self.refresh_count, error)
+
+  def cb_loading_complete(self, messages):
     self.refresh_count += 1
-    
     items = []
     errors = []
-    for o in output:
-      for o2 in o[1]:
-        if len(o2) > 1:
-          if o2[0] != "error":
-            with sqlite3.connect(SQLITE_DB_FILENAME) as db:
-              if len(db.execute("""select * from messages where mid = '%s' and account = '%s' and stream = '%s'""" % (o2[1], o2[2], o2[6])).fetchall()) > 0:
-                self.messages.Message("update", o2[-1])
-              else:
-                self.messages.Message("new", o2[-1])
-                for s in "messages", "replies", "private":
-                  if o2[6] == s and o2[9] != 1:
-                    self.unseen_counts[s] = self.unseen_counts[s] + 1
-            items.append(o2)
-          else:
-            errors.append(o2)
+    for m in messages:
+      if len(m) > 1:
+        if m[0] != "error":
+          with sqlite3.connect(SQLITE_DB_FILENAME) as db:
+            if len(db.execute("""select * from messages where mid = '%s' and account = '%s' and stream = '%s'""" % (m[1], m[2], m[6])).fetchall()) > 0:
+              self.messages.Message("update", m[-1])
+            else:
+              self.messages.Message("new", m[-1])
+              for s in "messages", "replies", "private":
+                if m[6] == s and m[9] != 1:
+                  self.unseen_counts[s] += 1
+            items.append(m)
+        else:
+          errors.append(m)
     with sqlite3.connect(SQLITE_DB_FILENAME) as db:
       oldid = db.execute("select max(ROWID) from messages").fetchone()[0] or 0
       
@@ -694,12 +728,14 @@ class Dispatcher(dbus.service.Object):
             ",".join(self.messages.columns),
             ",".join("?" * len(self.messages.columns))), items)
 
-      gobject.idle_add (self.update_indicators, self.unseen_counts)
-
-      new_items = db.execute("""
-        select * from (select * from messages where operation == "receive" and ROWID > %s and to_me = 0 ORDER BY time DESC LIMIT 10) as a union
-        select * from (select * from messages where operation IN ("receive","private") and ROWID > %s and to_me != 0 ORDER BY time DESC LIMIT 10) as b
-        ORDER BY time ASC""" % (oldid, oldid)).fetchall()
+      GLib.idle_add (self.update_indicators, self.unseen_counts)
+      query = """
+              SELECT * FROM messages WHERE rowid > {0} AND
+                                     ((operation == "receive" AND to_me = 0) OR
+                                     (operation IN ("receive", "private") AND to_me != 0))
+                                     ORDER BY time ASC LIMIT 10
+              """.format(oldid)
+      new_items = db.execute(query)
 
       for i in new_items:
           self.new_message(i)
@@ -718,37 +754,11 @@ class Dispatcher(dbus.service.Object):
       self.send_error_notify(error[1])
 
     self.LoadingComplete()
-    log.logger.info("Loading complete: %s - %s", self.refresh_count, [o[0] for o in output])
+    logger.info("Loading complete: %s - %s", self.refresh_count, output.rowcount if output.rowcount > 0 else 0)
 
   def update_indicators(self, counts):
     total_unseen = 0
     if indicate:
-      if counts.has_key("private"):
-        if not self.private_indicator:
-          self.private_indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
-          self.private_indicator.connect("user-display", self.on_indicator_activate)
-          self.private_indicator.set_property("name", _("Private"))
-          self.private_indicator.set_property("stream", "private")
-          self.private_indicator.show()
-        self.private_indicator.set_property("count", str(counts["private"]))
-        total_unseen +- counts["private"]
-        if counts["private"] > 0:
-          self.private_indicator.set_property_bool("draw-attention", True)
-        if self.private_indicator not in self.indicator_items:
-          self.indicator_items["private"] = self.private_indicator
-        log.logger.debug("Private Messages Indicator count updated to %s", counts["private"])
-      if counts.has_key("replies"):
-        if not self.replies_indicator:
-          self.replies_indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
-          self.replies_indicator.connect("user-display", self.on_indicator_activate)
-          self.replies_indicator.set_property("name", _("Replies"))
-          self.replies_indicator.set_property("stream", "replies")
-          self.replies_indicator.show()
-        self.replies_indicator.set_property("count", str(counts["replies"]))
-        total_unseen += counts["replies"]
-        if self.replies_indicator not in self.indicator_items:
-          self.indicator_items["replies"] = self.replies_indicator
-        log.logger.debug("Replies Indicator count updated to %s", counts["replies"])
       if counts.has_key("messages"):
         if not self.messages_indicator:
           self.messages_indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
@@ -760,9 +770,31 @@ class Dispatcher(dbus.service.Object):
         total_unseen += counts["messages"]
         if self.messages_indicator not in self.indicator_items:
           self.indicator_items["messages"] = self.messages_indicator
-        log.logger.debug("Messages Indicator count updated to %s", counts["messages"])
-    log.logger.debug ("Total unseen count: %d", total_unseen)
-    if Unity:
+      if counts.has_key("replies"):
+        if not self.replies_indicator:
+          self.replies_indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
+          self.replies_indicator.connect("user-display", self.on_indicator_activate)
+          self.replies_indicator.set_property("name", _("Replies"))
+          self.replies_indicator.set_property("stream", "replies")
+          self.replies_indicator.show()
+        self.replies_indicator.set_property("count", str(counts["replies"]))
+        total_unseen += counts["replies"]
+        if self.replies_indicator not in self.indicator_items:
+          self.indicator_items["replies"] = self.replies_indicator
+      if counts.has_key("private"):
+        if not self.private_indicator:
+          self.private_indicator = indicate.Indicator() if hasattr(indicate, "Indicator") else indicate.IndicatorMessage()
+          self.private_indicator.connect("user-display", self.on_indicator_activate)
+          self.private_indicator.set_property("name", _("Private"))
+          self.private_indicator.set_property("stream", "private")
+          self.private_indicator.show()
+        self.private_indicator.set_property("count", str(counts["private"]))
+        total_unseen += counts["private"]
+        if counts["private"] > 0:
+          self.private_indicator.set_property_bool("draw-attention", True)
+        if self.private_indicator not in self.indicator_items:
+          self.indicator_items["private"] = self.private_indicator
+    if Unity and self.launcher:
       self.launcher.set_property("count", total_unseen)
       if total_unseen < 1:
         self.launcher.set_property("count_visible", False)
@@ -779,7 +811,7 @@ class Dispatcher(dbus.service.Object):
   def on_indicator_server_activate(self, indicator, timestamp=None):
     dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
     client_bus = dbus.SessionBus()
-    log.logger.debug("Raising gwibber client")
+    logger.debug("Raising gwibber client")
     try:
       self.handle_indicator_counts()
     except:
@@ -789,7 +821,7 @@ class Dispatcher(dbus.service.Object):
   def on_indicator_activate(self, indicator, timestamp=None):
     if not indicate: return
     stream = indicator.get_property("stream")
-    log.logger.debug("Raising gwibber client, focusing %s stream", stream)
+    logger.debug("Raising gwibber client, focusing %s stream", stream)
     try:
       self.handle_indicator_counts(stream)
     except:
@@ -797,10 +829,10 @@ class Dispatcher(dbus.service.Object):
     self.show_client(stream=stream)
 
   def handle_focus_reply(self, *args):
-    log.logger.debug("Gwibber Client raised")
+    logger.debug("Gwibber Client raised")
 
   def handle_focus_error(self, *args):
-    log.logger.error("Failed to raise client %s", args)
+    logger.error("Failed to raise client %s", args)
 
   def handle_indicator_counts(self, stream=None):
     if indicate:
@@ -810,8 +842,9 @@ class Dispatcher(dbus.service.Object):
           self.unseen_counts[s] = 0
           if s == "private":
             self.private_indicator.set_property_bool("draw-attention", False)
-        self.launcher.set_property("count", 0)
-        self.launcher.set_property("count_visible", False)
+        if self.launcher:
+          self.launcher.set_property("count", 0)
+          self.launcher.set_property("count_visible", False)
         return
       if self.indicator_items.has_key(stream):
         self.indicator_items[stream].set_property("count", str(0))
@@ -823,33 +856,31 @@ class Dispatcher(dbus.service.Object):
       total_unseen = 0
       for s in self.unseen_counts.keys():
         total_unseen += self.unseen_counts[s]
-      log.logger.debug ("handle_indicator_counts total_unseen is %d", total_unseen)
-      self.launcher.set_property("count", total_unseen)
-      if total_unseen < 1:
-        self.launcher.set_property("count_visible", False)
-      else:
-        self.launcher.set_property("count_visible", True)
+      logger.debug ("handle_indicator_counts total_unseen is %d", total_unseen)
+      if self.launcher:
+        self.launcher.set_property("count", total_unseen)
+        if total_unseen < 1:
+          self.launcher.set_property("count_visible", False)
+        else:
+          self.launcher.set_property("count_visible", True)
     
 
   def new_search_message(self, data):
     message = json.loads(data[-1])
-    gobject.idle_add(self.cache_avatar, message)
+    GLib.idle_add(self.cache_avatar, message)
 
   def new_message(self, data):
     message = json.loads(data[-1])
     if message["transient"]:
-      log.logger.debug("Message %s is transient, not notifying", message["id"])
       return
 
     if util.can_notify and  str(message["mid"]) not in self.notified_items:
       self.notified_items.append(message["mid"])
       if gsettings.get_boolean("notify-mentions-only") and message["to_me"]: 
-        log.logger.debug("%s is a mention and notify_mentions_only is true", message["mid"])
-        gobject.idle_add(self.handle_notify_item, message)
+        GLib.idle_add(self.handle_notify_item, message)
       elif gsettings.get_boolean("show-notifications") and not gsettings.get_boolean("notify-mentions-only"):
-        log.logger.debug("%s - show_notifications is true and notify_mentions_only is false", message["mid"])
-        gobject.idle_add(self.handle_notify_item, message)
-    gobject.idle_add(self.cache_avatar, message)
+        GLib.idle_add(self.handle_notify_item, message)
+    GLib.idle_add(self.cache_avatar, message)
     
   def handle_notify_item(self, message):
     if gsettings.get_boolean("show-fullname"):
@@ -867,13 +898,14 @@ class Dispatcher(dbus.service.Object):
         notify_text = _("has shared a link")
       if message["stream"] == "videos":
         notify_text = _("has shared a video")
-    #image = util.resources.get_ui_asset("icons/breakdance/scalable/%s.svg" % message["service"])
     if message["sender"].has_key("image"):
       image = util.resources.get_avatar_path(message["sender"]["image"])
     else:
-      image = util.resources.get_ui_asset("icons/breakdance/scalable/%s.svg" % message["service"])
+      #FIXME - we need to fix finding service icons in the service
+      image = util.resources.get_ui_asset("%s.svg" % message["service"])
     if not image:
-      image = util.resources.get_ui_asset("icons/breakdance/scalable/%s.svg" % message["service"])
+      #FIXME - we need to fix finding service icons in the service
+      image = util.resources.get_ui_asset("%s.png" % message["service"])
     util.notify(sender_name, notify_text, image, 2000)
 
     return False
@@ -885,23 +917,24 @@ class Dispatcher(dbus.service.Object):
 
   def loading_failed(self, exception, tb):
     self.LoadingComplete()
-    log.logger.error("Loading failed: %s - %s", exception, tb)
+    logger.error("Loading failed: %s - %s", exception, tb)
 
   def send(self, operations):
     operations = util.compact(operations)
     if operations:
       self.LoadingStarted()
-      log.logger.debug("*** Sending Message ***")
+      logger.debug("*** Sending Message ***")
       self.perform_async_operation(operations)
 
   def refresh(self, *args):
+
     if self.refresh_timer_id:
-      gobject.source_remove(self.refresh_timer_id)
+      GLib.source_remove(self.refresh_timer_id)
 
     refresh_interval = gsettings.get_int("interval")
 
     if not self.maintRunning and not self.refreshRunning:
-      log.logger.debug("Refresh interval is set to %s", refresh_interval)
+      logger.debug("Refresh interval is set to %s", refresh_interval)
       operations = []
     
       for o in self.collector.get_operations():
@@ -910,17 +943,17 @@ class Dispatcher(dbus.service.Object):
           operations.append(o)
     
       if operations:
-        log.logger.debug("** Starting Refresh - %s **", mx.DateTime.now())
+        logger.debug("** Starting Refresh - %s **", mx.DateTime.now())
         self.LoadingStarted()
         self.perform_async_operation(operations)
 
 
-      self.refresh_timer_id = gobject.timeout_add_seconds(int(60 * refresh_interval), self.refresh)
+      self.refresh_timer_id = GLib.timeout_add_seconds(int(60 * refresh_interval), self.refresh)
     else:
-      self.refresh_timer_id = gobject.timeout_add_seconds(int(30), self.refresh)
+      self.refresh_timer_id = GLib.timeout_add_seconds(int(30), self.refresh)
 
     if not self.maintDone:
-      self.maint_timer_id = gobject.timeout_add_seconds(60, self.do_maintenance)
+      self.maint_timer_id = GLib.timeout_add_seconds(60, self.do_maintenance)
 
     return False
 
@@ -950,15 +983,15 @@ class ConnectionMonitor(dbus.service.Object):
 
 
     try:
-      log.logger.debug("NM Version is %s", str(self.nm.Get(NM_DBUS_INTERFACE, "Version")))
+      logger.debug("NM Version is %s", str(self.nm.Get(NM_DBUS_INTERFACE, "Version")))
       if str(self.nm.Get(NM_DBUS_INTERFACE, "Version")) >= "0.8.998":
-        log.logger.debug("NM Version is greater than 0.8.997")
+        logger.debug("NM Version is greater than 0.8.997")
         self.NM_STATE_ASLEEP = 10
         self.NM_STATE_DISCONNECTED = 20
         self.NM_STATE_CONNECTING = 40
         self.NM_STATE_CONNECTED = 70
       else:
-        log.logger.debug("NM Version is less than 0.8.998")
+        logger.debug("NM Version is less than 0.8.998")
         self.NM_STATE_ASLEEP = 1
         self.NM_STATE_CONNECTING = 2
         self.NM_STATE_CONNECTED = 3
@@ -969,13 +1002,13 @@ class ConnectionMonitor(dbus.service.Object):
 
 
   def on_connection_changed(self, state):
-    log.logger.debug("Network state changed, new state is %d", state)
+    logger.debug("Network state changed, new state is %d", state)
 
     if state == self.NM_STATE_CONNECTED:
-      log.logger.info("Network state changed to Online")
+      logger.info("Network state changed to Online")
       self.ConnectionOnline()
     elif state == self.NM_STATE_DISCONNECTED:
-      log.logger.info("Network state changed to Offline")
+      logger.info("Network state changed to Offline")
       self.ConnectionOffline()
 
   @dbus.service.signal("com.Gwibber.Connection")
@@ -987,7 +1020,7 @@ class ConnectionMonitor(dbus.service.Object):
   @dbus.service.method("com.Gwibber.Connection")
   def isConnected(self):
     if not self.has_nm: 
-      log.logger.info("Can't determine network state, assuming online")
+      logger.info("Can't determine network state, assuming online")
       return True
     try:
       if self.nm.state() == self.NM_STATE_CONNECTED:
@@ -1018,8 +1051,8 @@ class URLShorten(dbus.service.Object):
     """
     
     service = gsettings.get_string("urlshorter") or "is.gd"
-    log.logger.info("Shortening URL %s with %s", url, service)
-    if self.IsShort(url): return url
+    logger.info("Shortening URL %s with %s", url, service)
+    if self.IsShort(url) or not gsettings.get_boolean("shorten-urls"): return url
     try:
       s = urlshorter.PROTOCOLS[service].URLShorter()
       return s.short(url)
@@ -1065,16 +1098,16 @@ class Uploader(dbus.service.Object):
             sig_failed = uploader.connect_to_signal("UploadFailed", failed)
             uploader.Upload(filepath)
     """
-    log.logger.info("Uploading image %s", filepath)
+    logger.info("Uploading image %s", filepath)
     url = uploader.upload(filepath, self.UploadComplete, self.UploadFailed)
 
   @dbus.service.signal(dbus_interface="com.Gwibber.Uploader", signature="ss")
   def UploadComplete(self, filepath, public_url):
-    log.logger.info("Image %s uploaded as %s", filepath, public_url)
+    logger.info("Image %s uploaded as %s", filepath, public_url)
 
   @dbus.service.signal(dbus_interface="com.Gwibber.Uploader", signature="ss")
   def UploadFailed(self, filepath, error_message):
-    log.logger.info("Image %s failed to upload with message '%s'", filepath, error_message)
+    logger.info("Image %s failed to upload with message '%s'", filepath, error_message)
 
 class Translate(dbus.service.Object):
   __dbus_object_path__ = "/com/gwibber/Translate"
