@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 from gwibber.microblog import network, util
-from gwibber.microblog.util import log, resources
+from gwibber.microblog.util import resources
 import hashlib, mx.DateTime, time
 from os.path import join, getmtime, exists
 from gettext import lgettext as _
@@ -13,7 +13,9 @@ try:
 except:
   pass
 
-log.logger.name = "Facebook"
+import logging
+logger = logging.getLogger("Facebook")
+logger.debug("Initializing.")
 
 PROTOCOL_INFO = {
   "name": "Facebook",
@@ -39,6 +41,7 @@ PROTOCOL_INFO = {
     "delete",
     "send_thread",
     "like",
+    "unlike",
     "sincetime"
   ],
 
@@ -61,7 +64,7 @@ class Client:
   def _check_error(self, data):
     if isinstance(data, dict):
       if data.has_key("error"):
-        log.logger.info("Facebook error %s - %s", data["error"]["type"], data["error"]["message"])
+        logger.info("Facebook error %s - %s", data["error"]["type"], data["error"]["message"])
         return True
       else: 
         return False
@@ -70,7 +73,7 @@ class Client:
   def _get(self, operation, post=False, single=False, **args):
     if not self.user_id or "access_token" not in self.account:
       logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Authentication failed"), "Auth needs updating")
-      log.logger.error("%s", logstr)
+      logger.error("%s", logstr)
       return [{"error": {"type": "auth", "account": self.account, "message": _("Authentication failed, please re-authorize")}}]
 
     args.update({
@@ -86,15 +89,15 @@ class Client:
     if isinstance(data, dict) and data.get("error_msg", 0):
       if "permission" in data["error_msg"]:
         logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Authentication failed"), data["error_msg"])
-        log.logger.error("%s", logstr)
+        logger.error("%s", logstr)
         return [{"error": {"type": "auth", "account": self.account, "message": data["error_msg"]}}]
       elif data["error_code"] == 102:
         logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Session invalid"), data["error_msg"])
-        log.logger.error("%s", logstr)
+        logger.error("%s", logstr)
         return [{"error": {"type": "auth", "account": self.account, "message": data["error_msg"]}}]
       else:
         logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Unknown failure"), data["error_msg"])
-        log.logger.error("%s", logstr)
+        logger.error("%s", logstr)
         return [{"error": {"type": "unknown", "account": self.account, "message": data["error_msg"]}}]
 
     return data
@@ -111,7 +114,7 @@ class Client:
   
   def _message(self, data):
     if type(data) != dict:
-      log.logger.error("Cannot parse message data: %s", str(data))
+      logger.error("Cannot parse message data: %s", str(data))
       return {}
 
     m = {}
@@ -141,12 +144,27 @@ class Client:
 
     m["type"] = data["type"]
 
-    if data["type"] == "photo":
+    if data["type"] == "checkin":
+      m["location"] = {}
+      if isinstance(data["place"], dict):
+        m["location"]["id"] = data["place"]["id"]
+        m["location"]["name"] = data["place"].get("name", None)
+        if m["location"]["name"]:
+          m["html"] += " " + _("at") + " <p><b>%s</b></p>" % m["location"]["name"]
+          m["content"] = m["html"]
+          m["text"] += " "  + _("at") + " " + m["location"]["name"]
+        if data["place"].has_key("location"):
+          m["location"]["latitude"] = data["place"]["location"].get("latitude", None)
+          m["location"]["longitude"] = data["place"]["location"].get("longitude", None)
+          m["location"]["city"] = data["place"]["location"].get("city", None)
+          m["location"]["state"] = data["place"]["location"].get("state", None)
+          m["location"]["country"] = data["place"]["location"].get("country", None)
+    elif data["type"] == "photo":
       m["photo"] = {}
       m["photo"]["picture"] = data.get("picture", None)
       m["photo"]["url"] = data.get("link", None)
       m["photo"]["name"] = data.get("name", None)
-    if data["type"] == "video":
+    elif data["type"] == "video":
       m["video"] = {}
       m["video"]["picture"] = data.get("picture", None)
       m["video"]["source"] = data.get("source", None)
@@ -154,7 +172,7 @@ class Client:
       m["video"]["name"] = data.get("name", None)
       m["video"]["icon"] = data.get("icon", None)
       m["video"]["properties"] = data.get("properties", {})
-    if data["type"] == "link":
+    elif data["type"] == "link":
       m["link"] = {}
       m["link"]["picture"] = data.get("picture", None)
       m["link"]["name"] = data.get("name", None)
@@ -163,6 +181,29 @@ class Client:
       m["link"]["icon"] = data.get("icon", None)
       m["link"]["caption"] = data.get("caption", None)
       m["link"]["properties"] = data.get("properties", {})
+    elif data["type"] == "question":
+      m["question"] = {}
+      m["question"]["id"] = data["id"]
+      m["question"]["text"] = data.get("question", None)
+      if not m["question"]["text"]:
+        m["question"]["text"] = data.get("story", None)
+      if data.has_key("options") and isinstance(data["options"], dict):
+        if isinstance(data["options"]["data"], dict):
+          m["question"]["answers"] = []
+          for a in data["options"]["data"]:
+            answer = {}
+            answer["id"] = a.get("id", None)
+            answer["name"] = a.get("name", None)
+            answer["votes"] = a.get("votes", None)
+            m["question"]["answers"].append(answer)
+      if m["question"]["text"] and len(m["text"]) < 1:
+        m["text"] = m["question"]["text"]
+        m["html"] = m["text"]
+        m["content"] = m["html"]
+    elif data["type"] == "status":
+      pass
+    else:
+      logger.error ("facebook: unexpected type %s", data["type"])
     
     if data.has_key("privacy"):
       m["privacy"] = {}
@@ -175,9 +216,14 @@ class Client:
 
     if data.has_key("likes"):
       m["likes"] = {}
+      m["likes"]["liked"] = False
       if isinstance(data["likes"], dict):
         m["likes"]["count"] = data["likes"]["count"]
-        m["likes"]["data"] = data["likes"]["data"]
+        if data["likes"].has_key("data"):
+          m["likes"]["data"] = data["likes"]["data"]
+          for d in m["likes"]["data"]:
+            if self.user_id == str(d["id"]):
+              m["likes"]["liked"] = True
       else:
         m["likes"]["count"] = data["likes"]
 
@@ -209,43 +255,52 @@ class Client:
     else:
       since = int(mx.DateTime.DateTimeFromTicks(since).localtime())
 
-    data = self._get("me/home")
+    data = self._get("me/home", since=since, limit=100)
     
-    log.logger.debug("<STATS> facebook:receive account:%s since:%s size:%s",
+    logger.debug("<STATS> facebook:receive account:%s since:%s size:%s",
         self.account["id"], mx.DateTime.DateTimeFromTicks(since), len(str(data)))
     
     if not self._check_error(data):
       try:
         return [self._message(post) for post in data["data"]]
       except TypeError:
-        log.logger.error("<facebook:receive> failed to parse message data")
+        logger.error("<facebook:receive> failed to parse message data")
         return data
     else: return 
 
   def delete(self, message):
     result = self._get("stream.remove", post_id=message["mid"])
     if not result:
-      log.logger.error("<facebook:delete> failed")
+      logger.error("<facebook:delete> failed")
       return
     return []
 
   def like(self, message):
     result = self._get(message["mid"] + "/likes", post=True)
     if not result:
-      log.logger.error("<facebook:like> failed")
-      return
-    return []
+      logger.error("<facebook:like> failed")
+      return []
+    data = self.receive ()
+    return data
+
+  def unlike(self, message):
+    result = self._get(message["mid"] + "/likes", post=True, method="delete")
+    if not result:
+      logger.error("<facebook:like> failed")
+      return []
+    data = self.receive ()
+    return data
 
   def send(self, message):
     result = self._get("me/feed", message=message, status_includes_verb=True, post=True)
     if not result:
-      log.logger.error("<facebook:send> failed")
+      logger.error("<facebook:send> failed")
       return
     return []
 
   def send_thread(self, message, target):
     result = self._get(target["mid"] + "/comments", message=message, post=True)
     if not result:
-      log.logger.error("<facebook:send_thread> failed")
+      logger.error("<facebook:send_thread> failed")
       return
     return []
