@@ -147,7 +147,7 @@ class AccountWidget(Gtk.VBox):
       data = json.loads(urllib.urlopen("https://graph.facebook.com/me?access_token=" + self.account["access_token"]).read())
       if isinstance(data, dict):
         if data.has_key("id") and data.has_key("name"):
-          self.account["username"] = data["name"]
+          self.account["username"] = data["name"].encode("utf-8")
           self.account["uid"] = data["id"]
           saved = self.dialog.on_edit_account_save() 
       else:
@@ -158,7 +158,7 @@ class AccountWidget(Gtk.VBox):
           if isinstance(acct, dict):
             if acct.has_key("id") and acct.has_key("name"):
               self.account["uid"] = acct["id"]
-              self.account["username"] = acct["name"]
+              self.account["username"] = acct["name"].encode("utf-8")
               saved = self.dialog.on_edit_account_save()
           else:
             print "Failed"
