@@ -1,12 +1,15 @@
 from gwibber.microblog import network, util
-from gwibber.microblog.util import log, resources
-import urllib, urllib2, json, htmllib, re
+from gwibber.microblog.util import resources
+import json, re
 from gettext import lgettext as _
-log.logger.name = "Foursquare"
+
+import logging
+logger = logging.getLogger("FourSquare")
+logger.debug("Initializing.")
 
 PROTOCOL_INFO = {
   "name": "Foursquare",
-  "version": "3.0",
+  "version": "4.0",
   
   "config": [
     "private:secret_token",
@@ -51,16 +54,24 @@ class Client:
 
     shouttext = ""
     text = ""
+    
+    if data.has_key("shout"):
+      shout = data["shout"]
+      shout = shout.replace('& ', '&amp; ')
+        
     if data.has_key("venue"):
+      venuename = data["venue"]["name"]
+      venuename = venuename.replace('& ', '&amp; ')
+    
       if data.has_key("shout"):
-        shouttext += data["shout"] + "\n\n"
-        text += data["shout"] + "\n"
+        shouttext += shout + "\n\n"
+        text += shout + "\n"
       if data["venue"].has_key("id"):
         m["url"] = "https://foursquare.com/venue/%s" % data["venue"]["id"]
       else:
         m["url"] = "https://foursquare.com"
-      shouttext += "Checked in at <a href='" + m["url"] + "'>" + data["venue"]["name"] + "</a>"
-      text += "Checked in at " + data["venue"]["name"]
+      shouttext += "Checked in at <a href='" + m["url"] + "'>" + venuename + "</a>"
+      text += "Checked in at " + venuename
       if data["venue"]["location"].has_key("address"):
         shouttext += ", " + data["venue"]["location"]["address"]
         text += ", " + data["venue"]["location"]["address"]
@@ -78,8 +89,8 @@ class Client:
           shouttext += " for " + data["event"]["name"]
     else:
       if data.has_key("shout"):
-          shouttext += data["shout"] + "\n\n"
-          text += data["shout"] + "\n"
+          shouttext += shout + "\n\n"
+          text += shout + "\n"
       else:
           text= "Checked in off the grid"
           shouttext= "Checked in off the grid"
@@ -91,7 +102,7 @@ class Client:
     m["sender"] = {}
     m["sender"]["id"] = data["user"]["id"]
     m["sender"]["image"] = data["user"]["photo"]
-    m["sender"]["url"] = "https://foursquare.com/user/-%s" % data["user"]["id"]
+    m["sender"]["url"] = data["user"]["canonicalUrl"]
     if data["user"]["relationship"] == "self": 
         m["sender"]["is_me"] = True 
     else: 
@@ -103,7 +114,7 @@ class Client:
         fullname += data["user"]["lastName"]
 
     if data.has_key("photos"):
-      if data["photos"]["count"] == 1:
+      if data["photos"]["count"] > 0:
         m["photo"] = {}
         m["photo"]["url"] = ""
         m["photo"]["picture"] = data["photos"]["items"][0]["url"] 
@@ -117,6 +128,34 @@ class Client:
         m["source"] = "<a href='" + data["source"]["url"] + "'>" + data["source"]["name"] + "</a>"
     else:
         m["source"] = "<a href='https://foursquare.com/'>Foursquare</a>"
+    
+    if data.has_key("comments"):
+      if data["comments"]["count"] > 0:
+
+        m["comments"] = []
+        comments = self._get_comments(data["id"])
+        for comment in comments:
+          # Get the commenter's name
+          fullname = ""
+          if comment["user"].has_key("firstName"):
+            fullname += comment["user"]["firstName"] + " "
+          if comment["user"].has_key("lastName"):
+            fullname += comment["user"]["lastName"]
+         
+          # Create a sender
+          sender = {
+            "name" : fullname,
+            "id"   : comment["user"]["id"],
+            "is_me": False,
+            "image": comment["user"]["photo"],
+            "url"  : comment["user"]["canonicalUrl"]
+          }
+          # Create a comment
+          m["comments"].append({
+              "text"  : comment["text"],
+              "time"  : comment["createdAt"],
+              "sender": sender,
+          })
 
     return m
 
@@ -124,19 +163,28 @@ class Client:
     if isinstance(data, dict) and "recent" in data:
       return True
     else:
-      log.logger.error("Foursquare error %s", data)
+      logger.error("Foursquare error %s", data)
       return False
+  
+  def _get_comments(self, checkin_id):
+    url = "/".join((URL_PREFIX, "checkins", checkin_id))
+    url = url + "?oauth_token=" + self.token
+    data = network.Download(url, None, False).get_json()["response"]
+    return data["checkin"]["comments"]["items"]
 
   def _get(self, path, parse="message", post=False, single=False, **args):
     url = "/".join((URL_PREFIX, path))
     
     url = url + "?oauth_token=" + self.token
-    data = json.load(urllib2.urlopen(url))["response"]
+
+    data = network.Download(url, None, post).get_json()["response"]
+
+    resources.dump(self.account["service"], self.account["id"], data)
 
     if isinstance(data, dict) and data.get("errors", 0):
       if "authenticate" in data["errors"][0]["message"]:
         logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Authentication failed"), error["message"])
-        log.logger.error("%s", logstr)
+        logger.error("%s", logstr)
         return [{"error": {"type": "auth", "account": self.account, "message": data["errors"][0]["message"]}}]
       else:
         for error in data["errors"]:
@@ -145,11 +193,11 @@ class Client:
     elif isinstance(data, dict) and data.get("error", 0):
       if "Incorrect signature" in data["error"]:
         logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Request failed"), data["error"])
-        log.logger.error("%s", logstr)
+        logger.error("%s", logstr)
         return [{"error": {"type": "auth", "account": self.account, "message": data["error"]}}]
     elif isinstance(data, str):
       logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Request failed"), data)
-      log.logger.error("%s", logstr)
+      logger.error("%s", logstr)
       return [{"error": {"type": "request", "account": self.account, "message": data}}]
     if not self._check_error(data):
       return []
