@@ -120,7 +120,7 @@ class AccountWidget(Gtk.VBox):
          fullname += data["response"]["user"]["firstName"] + " "
       if data["response"]["user"].has_key("lastName"):
         fullname += data["response"]["user"]["lastName"]
-      self.account["username"] = fullname
+      self.account["username"] = fullname.encode("utf-8")
       self.account["user_id"] = data["response"]["user"]["id"]
 
       if isinstance(data, dict):
