@@ -1,10 +1,11 @@
-import re
 from gwibber.microblog import network, util
-import gnomekeyring
 from oauth import oauth
-from gwibber.microblog.util import log, resources
+from gwibber.microblog.util import resources
 from gettext import lgettext as _
-log.logger.name = "Identi.ca"
+
+import logging
+logger = logging.getLogger("Identica")
+logger.debug("Initializing.")
 
 PROTOCOL_INFO = {
   "name": "Identi.ca",
@@ -51,6 +52,11 @@ PROTOCOL_INFO = {
   ],
 }
 
+#
+# TODO: There is self.url_prefix and no global URL_PREFIX?
+#
+URL_PREFIX = "https://identi.ca"
+
 class Client:
   def __init__(self, acct):
     self.url_prefix = "https://identi.ca"
@@ -93,7 +99,7 @@ class Client:
         m["images"] = images
         m["type"] = "photo"
     except:
-      log.logger.error("%s failure - %s", PROTOCOL_INFO["name"], data)
+      logger.error("%s failure - %s", PROTOCOL_INFO["name"], data)
 
     return m
 
@@ -117,7 +123,7 @@ class Client:
 
   def _message(self, data):
     if type(data) != dict:
-      log.logger.error("Cannot parse message data: %s", str(data))
+      logger.error("Cannot parse message data: %s", str(data))
       return {}
 
     n = {}
@@ -208,7 +214,7 @@ class Client:
 
   def _get(self, path, parse="message", post=False, single=False, **args):
     if not self.account.has_key("access_token") and not self.account.has_key("secret_token"):
-      log.logger.error("%s unexpected result - %s", PROTOCOL_INFO["name"], _("Account needs to be re-authorized"))
+      logger.error("%s unexpected result - %s", PROTOCOL_INFO["name"], _("Account needs to be re-authorized"))
       return [{"error": {"type": "auth", "account": self.account, "message": _("Account needs to be re-authorized")}}]
 
     self.sigmethod = oauth.OAuthSignatureMethod_HMAC_SHA1()
@@ -229,24 +235,24 @@ class Client:
     resources.dump(self.account["service"], self.account["id"], data)
 
     if isinstance(data, dict) and data.get("error", 0):
-      log.logger.error("%s failure - %s", PROTOCOL_INFO["name"], data["error"])
+      logger.error("%s failure - %s", PROTOCOL_INFO["name"], data["error"])
       if "authenticate" in data["error"]:
         return [{"error": {"type": "auth", "account": self.account, "message": data["error"]}}]
       else:
         return [{"error": {"type": "unknown", "account": self.account, "message": data["error"]}}]
     elif isinstance(data, str):
-      log.logger.error("%s unexpected result - %s", PROTOCOL_INFO["name"], data)
+      logger.error("%s unexpected result - %s", PROTOCOL_INFO["name"], data)
       return [{"error": {"type": "unknown", "account": self.account, "message": data}}]
 
     if parse == "follow" or parse == "unfollow":
       if isinstance(data, dict) and data.get("error", 0):
-        logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Unable to %s user" % parse), data["error"])
-        log.logger.error("%s", logstr)
+        logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("%s failed" % parse), data["error"])
+        logger.error("%s", logstr)
         return [{"error": {"type": "auth", "account": self.account, "message": data["error"]}}]
       else:
         return [["friendships", {"type": parse, "account": self.account["id"], "service": self.account["service"],"user_id": data["id"], "nick": data["screen_name"]}]]
 
-    if parse == "profile":
+    if parse == "profile" and isinstance(data, dict):
       return self._profile(data)
 
     if single: return [getattr(self, "_%s" % parse)(data)]
@@ -260,7 +266,7 @@ class Client:
     data = data.get_json()
 
     if type(data) != dict:
-      log.logger.error("Cannot parse search data: %s", str(data))
+      logger.error("Cannot parse search data: %s", str(data))
       return []
 
     return [self._result(m) for m in data["results"]]
