@@ -8,6 +8,10 @@ SegPhault (Ryan Paul) - 11/09/2008
 
 from gwibber.microblog import network, util
 
+import logging
+logger = logging.getLogger("Ping.FM")
+logger.debug("Initializing.")
+
 PROTOCOL_INFO = {
   "name": "Ping.fm",
   "version": 0.1,
