@@ -1,10 +1,12 @@
 from gwibber.microblog import network, util
-import re
-import gnomekeyring
+import cgi
 from oauth import oauth
-from gwibber.microblog.util import log, resources
+from gwibber.microblog.util import resources
 from gettext import lgettext as _
-log.logger.name = "Twitter"
+
+import logging
+logger = logging.getLogger("Twitter")
+logger.debug("Initializing.")
 
 PROTOCOL_INFO = {
   "name": "Twitter",
@@ -57,7 +59,7 @@ PROTOCOL_INFO = {
 URL_PREFIX = "https://twitter.com"
 API_PREFIX = "https://api.twitter.com/1"
 
-class Client:
+class Client ():
   def __init__(self, acct):
     self.service = util.getbus("Service")
     if acct.has_key("secret_token") and acct.has_key("password"): acct.pop("password")
@@ -71,33 +73,129 @@ class Client:
     self.token = oauth.OAuthToken(acct["access_token"], acct["secret_token"])
 
   def _common(self, data):
-    m = {};
+    m = {}
     try:
-
       m["mid"] = str(data["id"])
       m["service"] = "twitter"
       m["account"] = self.account["id"]
       if data.has_key("created_at"):
         m["time"] = util.parsetime(data["created_at"])
       m["text"] = util.unescape(data["text"])
-      m["to_me"] = ("@%s" % self.account["username"]) in data["text"]
+      m["text"] = cgi.escape(m["text"])
+      m["content"] = m["text"]
+      
+      # Go through the entities in the tweet and use them to linkify/filter tweeks as appropriate
+      if data.has_key("entities"):
 
-      m["html"] = util.linkify(m["text"],
-        ((util.PARSE_HASH, '#<a class="hash" href="%s#search?q=\\1">\\1</a>' % URL_PREFIX),
-        (util.PARSE_NICK, '@<a class="nick" href="%s/\\1">\\1</a>' % URL_PREFIX)), escape=False)
+        #Get mention entries
+        if data["entities"].has_key("user_mentions"):
+          names = []
+          for mention in data["entities"]["user_mentions"]:
+            if not mention["screen_name"] in names:
+              try:
+                m["content"] = m["content"].replace("@" + mention["screen_name"], "@<a href='gwibber:/user?acct=" + m["account"] + "&name=@" + mention["screen_name"] + "'>" + mention["screen_name"] + "</a>")
+              except:
+                pass
+            names.append(mention["screen_name"])  
 
-      m["content"] = util.linkify(m["text"],
-        ((util.PARSE_HASH, '#<a href="gwibber:/tag?acct=%s&query=\\1">\\1</a>' % m["account"]),
-        (util.PARSE_NICK, '@<a href="gwibber:/user?acct=%s&name=\\1">\\1</a>' % m["account"])), escape=True)
+        #Get hashtag entities
+        if data["entities"].has_key("hashtags"):
+          hashtags = []
+          for tag in data["entities"]["hashtags"]:
+            if not tag["text"] in hashtags:
+              try:
+                m["content"] = m["content"].replace("#" + tag["text"], "#<a href='gwibber:/tag?acct=" + m["account"] + "&query=#" + tag["text"] + "'>" + tag["text"] + "</a>")
+              except:
+                pass
+            hashtags.append(tag["text"])
+        
+        # Get url entities - These usually go in the link stream, but if they're picturesor videos, they should go in the proper stream
+        if data["entities"].has_key("urls"):
+          for urls in data["entities"]["urls"]:
+              url = cgi.escape (urls["url"])
+              expanded_url = url
+              if urls.has_key("expanded_url"):
+                if not urls["expanded_url"] is None:
+                  expanded_url = cgi.escape(urls["expanded_url"])
 
-      m["favorited"] = data.get("favorited", False)
+              display_url  = url
+              if urls.has_key("display_url"):
+                display_url = cgi.escape (urls["display_url"])
+ 
+              if url == m["content"]:
+                m["content"] = "<a href='" + url + "' title='" + expanded_url + "'>" + display_url + "</a>"
+              else:
+                try:
+                  startindex = m["content"].index(url)
+                  endindex   = startindex + len(url)
+                  start      = m["content"][0:startindex]
+                  end        = m["content"][endindex:]
+                  m["content"] = start + "<a href='" + url + "' title='" + expanded_url + "'>" + display_url + "</a>" + end
+                except:
+                  logger.debug ("Failed to set url for ID: %s",  m["mid"])
+
+              m["type"] = "link"
+
+              images = util.imgpreview(expanded_url)
+              videos = util.videopreview(expanded_url)
+              if images:
+                m["images"] = images
+                m["type"] = "photo"
+              elif videos:
+                m["images"] = videos
+                m["type"] = "video"
+              else:
+                # Well, it's not anything else, so it must be a link
+                m["link"] = {}
+                m["link"]["picture"] = ""
+                m["link"]["name"] = ""
+                m["link"]["description"] = m["content"]
+                m["link"]["url"] = url
+                m["link"]["icon"] = ""
+                m["link"]["caption"] = ""
+                m["link"]["properties"] = {}
+              
+        if data["entities"].has_key("media"):
+          for media in data["entities"]["media"]:
+            try:
+              url = cgi.escape (media["url"])
+              media_url_https = media["media_url_https"]
+              expanded_url = url
+              if media.has_key("expanded_url"):
+                expanded_url = cgi.escape(media["expanded_url"])
+                
+              display_url  = url
+              if media.has_key("display_url"):
+                display_url = cgi.escape (media["display_url"])
+ 
+              startindex = m["content"].index(url)
+              endindex   = startindex + len(url)
+              start      = m["content"][0:startindex]
+              end        = m["content"][endindex:]
+              m["content"] = start + "<a href='" + url + "' title='" + expanded_url  + "'>" + display_url + "</a>" + end
+              
+              if media["type"] == "photo":
+                m["type"] = "photo"
+                m["photo"] = {}
+                m["photo"]["picture"] = media_url_https
+                m["photo"]["url"] = None
+                m["photo"]["name"] = None
+
+            except:
+              pass
+
+      else:
+        m["content"] = util.linkify(util.unescape(m["text"]),
+          ((util.PARSE_HASH, '#<a href="gwibber:/tag?acct=%s&query=\\1">\\1</a>' % m["account"]),
+          (util.PARSE_NICK, '@<a href="gwibber:/user?acct=%s&name=\\1">\\1</a>' % m["account"])), escape=True)
+
+      m["html"] = m["content"]
+      
+      m["to_me"] = ("@%s" % self.account["username"]) in data["text"]   # Check if it's a reply directed at the user
+      m["favorited"] = data.get("favorited", False)                     # Check if the tweet has been favourited
 
-      images = util.imgpreview(m["text"])
-      if images:
-        m["images"] = images
-        m["type"] = "photo"
     except: 
-      log.logger.error("%s failure - %s", PROTOCOL_INFO["name"], data)
+      logger.error("%s failure - %s", PROTOCOL_INFO["name"], data)
       return {}
  
     return m
@@ -122,7 +220,7 @@ class Client:
     
   def _message(self, data):
     if type(data) != dict:
-      log.logger.error("Cannot parse message data: %s", str(data))
+      logger.error("Cannot parse message data: %s", str(data))
       return {}
 
     n = {}
@@ -153,13 +251,20 @@ class Client:
           m["reply"]["url"] = None
 
     m["sender"] = self._user(data["user"] if "user" in data else data["sender"])
-    m["url"] = "/".join((m["sender"]["url"], "statuses", str(m["mid"])))
+    m["url"] = "/".join((m["sender"]["url"], "statuses", str(m.get("mid", None))))
+
+    return m
+
+  def _responses(self, data):
+    m = self._message(data)
+    m["type"] = None
 
     return m
 
   def _private(self, data):
     m = self._message(data)
     m["private"] = True
+    m["type"] = None
 
     m["recipient"] = {}
     m["recipient"]["name"] = data["recipient"]["name"]
@@ -247,18 +352,18 @@ class Client:
     request = oauth.OAuthRequest.from_consumer_and_token(self.consumer, self.token,
         http_method=post and "POST" or "GET", http_url=url, parameters=util.compact(args))
     request.sign_request(self.sigmethod, self.consumer, self.token)
-    
+
     if post:
-      data = network.Download(request.to_url(), util.compact(args), post).get_json()
+      headers = request.to_header()
+      data = network.Download(url, util.compact(args), post, header=headers).get_json()
     else:
       data = network.Download(request.to_url(), None, post).get_json()
-
     resources.dump(self.account["service"], self.account["id"], data)
 
     if isinstance(data, dict) and data.get("errors", 0):
       if "authenticate" in data["errors"][0]["message"]:
-        logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Authentication failed"), error["message"])
-        log.logger.error("%s", logstr)
+        logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Authentication failed"), data["errors"][0]["message"])
+        logger.error("%s", logstr)
         return [{"error": {"type": "auth", "account": self.account, "message": data["errors"][0]["message"]}}]
       else:
         for error in data["errors"]:
@@ -267,22 +372,22 @@ class Client:
     elif isinstance(data, dict) and data.get("error", 0):
       if "Incorrect signature" in data["error"]:
         logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Request failed"), data["error"])
-        log.logger.error("%s", logstr)
+        logger.error("%s", logstr)
         return [{"error": {"type": "auth", "account": self.account, "message": data["error"]}}]
     elif isinstance(data, str):
       logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Request failed"), data)
-      log.logger.error("%s", logstr)
+      logger.error("%s", logstr)
       return [{"error": {"type": "request", "account": self.account, "message": data}}]
 
     if parse == "follow" or parse == "unfollow":
       if isinstance(data, dict) and data.get("error", 0):
-        logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("Unable to %s user" % parse), data["error"])
-        log.logger.error("%s", logstr)
+        logstr = """%s: %s - %s""" % (PROTOCOL_INFO["name"], _("%s failed" % parse), data["error"])
+        logger.error("%s", logstr)
         return [{"error": {"type": "auth", "account": self.account, "message": data["error"]}}]
       else:
         return [["friendships", {"type": parse, "account": self.account["id"], "service": self.account["service"],"user_id": data["id"], "nick": data["screen_name"]}]]
     
-    if parse == "profile":
+    if parse == "profile" and isinstance(data, dict):
       return self._profile(data)
 
     if parse == "list":
@@ -297,7 +402,7 @@ class Client:
     data = data.get_json()["results"]
 
     if type(data) != list:
-      log.logger.error("Cannot parse search data: %s", str(data))
+      logger.error("Cannot parse search data: %s", str(data))
       return []
 
     return [self._result(m) for m in data]
@@ -306,18 +411,18 @@ class Client:
     return getattr(self, opname)(**args)
   
   def receive(self, count=util.COUNT, since=None):
-    return self._get("statuses/home_timeline.json", count=count, since_id=since)
+    return self._get("statuses/home_timeline.json", include_entities=1, count=count, since_id=since)
 
   def responses(self, count=util.COUNT, since=None):
-    return self._get("statuses/mentions.json", count=count, since_id=since)
+    return self._get("statuses/mentions.json", "responses", include_entities=1, count=count, since_id=since)
 
   def private(self, count=util.COUNT, since=None):
-    private = self._get("direct_messages.json", "private", count=count, since_id=since) or []
+    private = self._get("direct_messages.json", "private", include_entities=1, count=count, since_id=since) or []
     private_sent = self._get("direct_messages/sent.json", "private", count=count, since_id=since) or []
     return private + private_sent
 
   def public(self):
-    return self._get("statuses/public_timeline.json")
+    return self._get("statuses/public_timeline.json", include_entities=1)
 
   def lists(self, **args):
     following = self._get("%s/lists/subscriptions.json" % self.account["username"], "list") or []
@@ -325,10 +430,10 @@ class Client:
     return following + lists
 
   def list(self, user, id, count=util.COUNT, since=None):
-    return self._get("%s/lists/%s/statuses.json" % (user, id), per_page=count, since_id=since)
+    return self._get("%s/lists/%s/statuses.json" % (user, id), include_entities=1, per_page=count, since_id=since)
 
   def search(self, query, count=util.COUNT, since=None):
-    return self._search(q=query, rpp=count, since_id=since)
+    return self._search(include_entities=1, q=query, rpp=count, since_id=since)
 
   def tag(self, query, count=util.COUNT, since=None):
     return self._search(q="#%s" % query, count=count, since_id=since)
@@ -365,5 +470,5 @@ class Client:
 
   def user_messages(self, id=None, count=util.COUNT, since=None):
     profiles = [self.profile(id)] or []
-    messages = self._get("statuses/user_timeline.json", id=id, count=count, since_id=since) or []
+    messages = self._get("statuses/user_timeline.json", id=id, include_entities=1, count=count, since_id=since) or []
     return messages + profiles
