@@ -1,42 +1,41 @@
-import gtk, pango, webkit, gnomekeyring
+from gi.repository import GLib, Gtk, Pango, WebKit
 import urllib, urllib2, json, urlparse, uuid
 from oauth import oauth
 
-from gtk import Builder
 from gwibber.microblog.util import resources
+from gwibber.microblog.util.keyring import get_from_keyring
 import gettext
 from gettext import gettext as _
 if hasattr(gettext, 'bind_textdomain_codeset'):
     gettext.bind_textdomain_codeset('gwibber','UTF-8')
 gettext.textdomain('gwibber')
 
-gtk.gdk.threads_init()
+GLib.threads_init()
 
 sigmeth = oauth.OAuthSignatureMethod_HMAC_SHA1()
 
-class AccountWidget(gtk.VBox):
+class AccountWidget(Gtk.VBox):
   """AccountWidget: A widget that provides a user interface for configuring twitter accounts in Gwibber
   """
   
   def __init__(self, account=None, dialog=None):
     """Creates the account pane for configuring Twitter accounts"""
-    gtk.VBox.__init__( self, False, 20 )
-    self.ui = gtk.Builder()
+    Gtk.VBox.__init__( self, False, 20 )
+    self.ui = Gtk.Builder()
     self.ui.set_translation_domain ("gwibber")
     self.ui.add_from_file (resources.get_ui_asset("gwibber-accounts-twitter.ui"))
     self.ui.connect_signals(self)
     self.vbox_settings = self.ui.get_object("vbox_settings")
-    self.pack_start(self.vbox_settings, False, False)
+    self.pack_start(self.vbox_settings, True, True, 0)
     self.show_all()
 
     self.account = account or {}
     self.dialog = dialog
-    has_secret_key = True
+    self.window = dialog.dialog
+    has_secret_key = False
     if self.account.has_key("id"):
-      try:
-        value = gnomekeyring.find_items_sync(gnomekeyring.ITEM_GENERIC_SECRET, {"id": str("%s/%s" % (self.account["id"], "secret_token"))})[0].secret
-      except gnomekeyring.NoMatchError:
-        has_secret_key = False
+        has_secret_key = get_from_keyring(self.account['id'],
+                                          'secret_token') is not None
 
     try:
       if self.account.has_key("access_token") and self.account.has_key("secret_token") and self.account.has_key("username") and has_secret_key and not self.dialog.condition:
@@ -53,11 +52,16 @@ class AccountWidget(gtk.VBox):
         self.dialog.ui.get_object("vbox_create").hide()
 
 
+  def console_message_cb (self, *args):
+    return True
+
   def on_twitter_auth_clicked(self, widget, data=None):
     self.winsize = self.window.get_size()
 
-    web = webkit.WebView()
+    web = WebKit.WebView()
     web.get_settings().set_property("enable-plugins", False)
+    web.get_settings().set_property("enable-developer-extras", False)
+    web.connect("console-message", self.console_message_cb)
     web.load_html_string(_("<p>Please wait...</p>"), "file:///")
 
     self.consumer = oauth.OAuthConsumer(*resources.get_twitter_keys())
@@ -74,35 +78,41 @@ class AccountWidget(gtk.VBox):
     url = "http://api.twitter.com/oauth/authorize?oauth_token=" + self.token.key
 
     web.load_uri(url)
-    web.set_size_request(550, 450)
     web.connect("title-changed", self.on_twitter_auth_title_change)
 
-    self.scroll = gtk.ScrolledWindow()
+    self.scroll = Gtk.ScrolledWindow()
+    self.scroll.set_size_request(550, 450)
     self.scroll.add(web)
 
     self.pack_start(self.scroll, True, True, 0)
     self.show_all()
+    self.dialog.infobar.hide()
 
     self.ui.get_object("vbox1").hide()
     self.ui.get_object("vbox_advanced").hide()
-    self.dialog.infobar.set_message_type(gtk.MESSAGE_INFO)
+    self.dialog.infobar.set_message_type(Gtk.MessageType.INFO)
 
   def on_twitter_auth_title_change(self, web=None, title=None, data=None):
     saved = False
+    uri = title.get_uri ()
+    if hasattr(self.dialog, "infobar_content_area"):
+      for child in self.dialog.infobar_content_area.get_children(): child.destroy()
+    self.dialog.infobar_content_area = self.dialog.infobar.get_content_area()
+    self.dialog.infobar_content_area.show()
+    #self.dialog.infobar.show()
+
     if title.get_title() == "Success":
+      if "denied" in uri:
+        self.scroll.destroy()
+        self.window.resize(*self.winsize)
+        return
 
-      if hasattr(self.dialog, "infobar_content_area"):
-        for child in self.dialog.infobar_content_area.get_children(): child.destroy()
-      self.dialog.infobar_content_area = self.dialog.infobar.get_content_area()
-      self.dialog.infobar_content_area.show()
-      self.dialog.infobar.show()
-
-      message_label = gtk.Label(_("Verifying"))
+      message_label = Gtk.Label (_("Verifying"))
       message_label.set_use_markup(True)
-      message_label.set_ellipsize(pango.ELLIPSIZE_END)
+      message_label.set_ellipsize(Pango.EllipsizeMode.END)
       self.dialog.infobar_content_area.add(message_label)
       self.dialog.infobar.show_all()
-      self.scroll.hide()
+      self.scroll.destroy()
       url = web.get_main_frame().get_uri()
       data = urlparse.parse_qs(url.split("?", 1)[1])
 
@@ -142,16 +152,18 @@ class AccountWidget(gtk.VBox):
           saved = self.dialog.on_edit_account_save()
         else:
           print "Failed"
-          self.dialog.infobar.set_message_type(gtk.MESSAGE_ERROR)
-          message_label.set_text(_("Authorization failed. Please try again.")) 
+          self.dialog.infobar.set_message_type(Gtk.MessageType.ERROR)
+          message_label.set_text (_("Authorization failed. Please try again."))
+          self.dialog.infobar_content_area.add(message_label)
+          self.dialog.infobar.show_all()
       else:
         print "Failed"
-        self.dialog.infobar.set_message_type(gtk.MESSAGE_ERROR)
+        self.dialog.infobar.set_message_type(Gtk.MessageType.ERROR)
         message_label.set_text(_("Authorization failed. Please try again."))
 
       if saved: 
         message_label.set_text(_("Successful"))
-        self.dialog.infobar.set_message_type(gtk.MESSAGE_INFO)
+        self.dialog.infobar.set_message_type(Gtk.MessageType.INFO)
         #self.dialog.infobar.hide()
 
       self.ui.get_object("hbox_twitter_auth").hide()
@@ -162,12 +174,17 @@ class AccountWidget(gtk.VBox):
       elif self.dialog.ui and not saved:
         self.dialog.ui.get_object("vbox_create").show()
 
-    self.window.resize(*self.winsize)
+      self.window.resize(*self.winsize)
 
     if title.get_title() == "Failure":
-      web.hide()
-      self.dialog.infobar.set_message_type(gtk.MESSAGE_ERROR)
-      message_label.set_text(_("Authorization failed. Please try again."))
+      self.scroll.destroy()
+      self.window.resize(*self.winsize)
+      self.scroll.destroy()
+      self.dialog.infobar.set_message_type(Gtk.MessageType.ERROR)
+      message_label = Gtk.Label (_("Authorization failed. Please try again."))
+      message_label.set_use_markup(True)
+      message_label.set_ellipsize(Pango.EllipsizeMode.END)
+      self.dialog.infobar_content_area.add(message_label)
       self.dialog.infobar.show_all()
 
       self.ui.get_object("vbox1").show()
