@@ -233,14 +233,14 @@ try:
     pixbuf = None
     try:
       pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(icon, iconsize, iconsize)
-    except GLib.Error as e:
+    except GLib.GError as e:
       logger.error("Avatar failure - %s - %s", icon, e.message)
       resources.del_avatar(icon)
 
     if pixbuf:
       try:
         notification.set_icon_from_pixbuf(pixbuf)
-      except GLib.Error as e:
+      except GLib.GError as e:
         logger.error("Notification failure - %s", e.message)
 
     if timeout:
