@@ -1,7 +1,9 @@
-import log
 import os, subprocess
 import xdg, time
 
+import logging
+logger = logging.getLogger("Exceptions")
+
 import gettext
 from gettext import lgettext as _
 if hasattr(gettext, 'bind_textdomain_codeset'):
@@ -23,15 +25,15 @@ class GwibberServiceError(GwibberError):
     """
     def __init__(self, kind="UNKNOWN", service="UNKNOWN", username="UNKNOWN", message="UNKNOWN"):
         if kind == "keyring" or kind == "auth":
-            log.logger.error("Failed to find credentials in the keyring")
+            logger.error("Failed to find credentials in the keyring")
             accounts_error = os.path.join(xdg.BaseDirectory.xdg_cache_home, "gwibber", ".accounts_error")
             if os.path.exists(accounts_error) and os.path.getmtime(accounts_error) > time.time()-600:
-                log.logger.info("gwibber-accounts was raised less than 600 seconds")
+                logger.info("gwibber-accounts was raised less than 600 seconds")
                 return
             else:
                 open(accounts_error, 'w').close() 
         else:
-            log.logger.error("%s failure: %s:%s - %s", kind, service, username, message)
+            logger.error("%s failure: %s:%s - %s", kind, service, username, message)
 
         display_message = _("There was an %(kind)s failure from %(service)s for account %(account)s, error was %(error)s") % {
           "kind": kind, 
