@@ -5,16 +5,18 @@ SegPhault (Ryan Paul) - 11/22/2008
 """
 
 import os, sys
+from hashlib import sha1
+from os import makedirs, remove, environ
 from os.path import join, isdir, realpath, exists
-from os import makedirs, remove
-from os import environ
 import Image
-import log
 import mx.DateTime
 from gwibber.microblog import network
-from const import *
+from gwibber.microblog.util.const import *
 import inspect
 
+import logging
+logger = logging.getLogger("Resources")
+
 # Try to import * from custom, install custom.py to include packaging 
 # customizations like distro API keys, etc
 try:
@@ -22,8 +24,6 @@ try:
 except:
   pass
 
-log.logger.name = "Gwibber Dispatcher Resources"
-
 PROGRAM_NAME = "gwibber"
 UI_DIR_NAME = "ui"
 THEME_DIR_NAME = os.path.join(UI_DIR_NAME, "themes")
@@ -37,9 +37,9 @@ THEME_NAME = "default"
 THEME_MIN_VERSION = 2
 
 try:
-  import xdg
-  DATA_BASE_DIRS = xdg.BaseDirectory.xdg_data_dirs
-  CACHE_BASE_DIR = xdg.BaseDirectory.xdg_cache_home
+  from xdg import BaseDirectory
+  DATA_BASE_DIRS = BaseDirectory.xdg_data_dirs
+  CACHE_BASE_DIR = BaseDirectory.xdg_cache_home
 except:
   DATA_BASE_DIRS = [
     os.path.join(os.path.expanduser("~"), ".local", "share"),
@@ -66,6 +66,10 @@ def get_plugin_dirs():
 
   PLUGINS_DIRS.reverse()
 
+  if environ.has_key("GWIBBER_PLUGIN_TEST_DIR"):
+    PLUGINS_DIRS = []
+    PLUGINS_DIRS.insert (0, os.path.realpath(environ["GWIBBER_PLUGIN_TEST_DIR"]))
+
   for p in PLUGINS_DIRS:
     if exists(p):
       sys.path.insert(0, p)
@@ -83,10 +87,10 @@ def get_avatar_path(url):
   avatar_cache_dir = realpath(join(CACHE_BASE_DIR, "gwibber", "avatars"))
   if not isdir(avatar_cache_dir):
     makedirs(avatar_cache_dir)
-  avatar_cache_image = join(avatar_cache_dir, url.replace("/",""))
+  avatar_cache_image = join(avatar_cache_dir, sha1(url).hexdigest())
 
   if not exists(avatar_cache_image) or len(open(avatar_cache_image, "r").read()) < 1:
-    log.logger.debug("Downloading avatar %s", url)
+    logger.debug("Downloading avatar %s", url)
     f = file(avatar_cache_image, "w")
     data = network.Download(url)
     f.write(data.get_string())
@@ -102,7 +106,7 @@ def del_avatar(avatar):
     try:
       remove(avatar)
     except:
-      log.logger.error("Failed to remove avatar from cache: %s", avatar)
+      logger.error("Failed to remove avatar from cache: %s", avatar)
 
 def img_resize(img_path, size):
   try:
@@ -114,7 +118,7 @@ def img_resize(img_path, size):
       image.resize((size, size), Image.ANTIALIAS).save(img_path, format="jpeg")
   except Exception, e:
     from traceback import format_exc
-    log.logger.error("Image resizing failed:\n%s", format_exc())
+    logger.error("Image resizing failed:\n%s", format_exc())
 
 def get_desktop_file():
   p = os.path.join(LAUNCH_DIR, "gwibber.desktop")
@@ -168,9 +172,17 @@ def get_ui_asset(asset_name):
             return asset_path
 
   for base in DATA_DIRS:
+    asset_path = os.path.join(base, 'data', asset_name)
+    if os.path.exists(asset_path):
+      return asset_path
+
     asset_path = os.path.join(base, UI_DIR_NAME, asset_name)
     if os.path.exists(asset_path):
       return asset_path
+
+    asset_path = os.path.join(base, asset_name)
+    if os.path.exists(asset_path):
+      return asset_path
   return None
 
 def get_template_dirs():
@@ -202,7 +214,7 @@ def dump(service, aid, data):
     dump_cache_file = join(dump_cache_dir, (aid + "." + str(mx.DateTime.now()) + "." + operation))
 
     if not exists(dump_cache_file) or len(open(dump_cache_file, "r").read()) < 1:
-      log.logger.debug("Dumping test data %s - %s - %s", service, aid, operation)
+      logger.debug("Dumping test data %s - %s - %s", service, aid, operation)
       f = file(dump_cache_file, "w")
       f.write(str(data))
       f.close()
