@@ -1,6 +1,6 @@
-import gtk, dbus, os, mx.DateTime, webbrowser
+import dbus, os, mx.DateTime, webbrowser
 import subprocess
-import gobject
+from gi.repository import GLib, Gdk, GdkPixbuf, Gtk
 from microblog.util import resources
 from microblog.util.const import *
 # Try to import * from custom, install custom.py to include packaging 
@@ -31,29 +31,29 @@ class Color:
 
   @classmethod
   def from_gtk_color(self, c):
-    if isinstance(c, gtk.gdk.Color): c = c.to_string()
+    if isinstance(c, Gdk.Color): c = c.to_string()
     return self("#" + "".join([c[1:3], c[5:7], c[9:11]]))
 
 def get_style():
-  w = gtk.Window()
+  w = Gtk.Window()
   w.realize()
   return w.get_style()
 
-def get_theme_colors(w = gtk.Window()):
+def get_theme_colors(w = Gtk.Window()):
   w.realize()
   style = w.get_style()
   output = {}
   
   for i in ["base", "text", "fg", "bg"]:
-    c = getattr(style, i)[gtk.STATE_NORMAL]
+    c = getattr(style, i)[Gtk.StateType.NORMAL]
     output[i] = Color.from_gtk_color(c)
-    c = getattr(style, i)[gtk.STATE_SELECTED]
+    c = getattr(style, i)[Gtk.StateType.SELECTED]
     output["%s_selected" % i] = Color.from_gtk_color(c)
 
   return output
 
 def pixbuf(path):
-  return gtk.gdk.pixbuf_new_from_file(resources.get_ui_asset(path))
+  return GdkPixbuf.Pixbuf.new_from_file(resources.get_ui_asset(path))
 
 def icon(name, size=16, use_theme=True):
   if use_theme:
@@ -64,12 +64,12 @@ def icon(name, size=16, use_theme=True):
         if os.path.exists(fname + ext):
           return fname + ext
 
-  theme = gtk.icon_theme_get_default()
+  theme = Gtk.IconTheme.get_default()
   finfo = theme.lookup_icon(name, size, 0)
   return finfo.get_filename() if finfo else None
 
 def get_service_icon(name, size=16):
-  theme = gtk.icon_theme_get_default()
+  theme = Gtk.IconTheme.get_default()
   finfo = theme.lookup_icon(name, size, 0)
   if finfo:
     return finfo.get_filename()
@@ -90,7 +90,8 @@ def remove_urls(s):
     if not x.startswith('http://') and not x.startswith("https://"))
 
 try:
-  import gtkspell
+  #import gtkspell
+  gtkspell = None
 except:
   gtkspell = None
 
@@ -142,7 +143,7 @@ def show_accounts_dialog(account=None, message=None, obj=None, callback=None):
     cmd.insert(0, "gwibber-accounts")
   print cmd
   process = subprocess.Popen(cmd)
-  gobject.timeout_add(500, check_ret, process, obj, callback) 
+  GLib.timeout_add(500, check_ret, process, obj, callback) 
 
 def check_ret(process, obj=None, callback=None):
   if callback == None:
